@@ -26,6 +26,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <sys/socket.h>  /* For have_ipv6() */
 #include <netdb.h>
 
 #include <gconf/gconf-client.h>
@@ -48,6 +49,7 @@
 #include "util.h"
 #include "gsm-sound.h"
 #include "gsm-gsd.h"
+#include "gsm-at-startup.h"
 
 /* Flag indicating autosave - user won't be prompted on logout to 
  * save the session */
@@ -136,6 +138,19 @@ set_lang (void)
     }
 }
 
+/* Point GTK_RC_FILES at a separate file that we change in
+ * in gnome-settings-daemon.
+ */
+static void
+set_gtk1_theme_rcfile (void)
+{
+  char *env_string;
+
+  env_string = g_strdup_printf ("GTK_RC_FILES=" SYSCONFDIR "/gtk/gtkrc:%s/.gtkrc-1.2-gnome2", g_get_home_dir ());
+
+  putenv (env_string);
+}
+
 static void
 update_boolean (GConfClient *client,
 		guint cnxn_id,
@@ -176,6 +191,22 @@ get_hostname (gboolean readable)
     return readable ? "(Unknown)" : NULL;
 }
 
+#ifdef ENABLE_IPV6
+/*Check whether the node is IPv6 enabled.*/
+static gboolean
+have_ipv6 (void)
+{
+  int s;
+
+  s = socket (AF_INET6, SOCK_STREAM, 0);
+  if (s != -1) {
+    close (s);
+    return TRUE;
+  }
+  return FALSE;
+}
+#endif
+
 /* Check if a DNS lookup on `hostname` can be done */
 static gboolean
 check_for_dns (void)
@@ -187,13 +218,32 @@ check_for_dns (void)
   if (!hostname)
     return FALSE;
 
-  /*
-   * FIXME:
-   *  we should probably be a lot more robust here
-   */
-  if (!gethostbyname (hostname))
-    return FALSE;
-  
+#ifdef ENABLE_IPV6
+  if (have_ipv6 ())
+    {
+      struct addrinfo hints, *result = NULL;
+
+      memset (&hints, 0, sizeof(hints));
+      hints.ai_socktype = SOCK_STREAM;
+      hints.ai_flags = AI_CANONNAME;
+
+      if (getaddrinfo (hostname, NULL, &hints, &result) != 0)
+	return FALSE;
+
+      if (g_strncasecmp (result->ai_canonname, hostname, 0) != 0)
+	return FALSE;
+    } 
+  else
+#endif
+    {
+      /*
+       * FIXME:
+       *  we should probably be a lot more robust here
+       */
+      if (!gethostbyname (hostname))
+	return FALSE;
+    }
+
   return TRUE;
 }
 
@@ -439,6 +489,7 @@ main (int argc, char *argv[])
   Session *the_session;
   GConfClient *gconf_client;
   gboolean splashing;
+  gboolean a_t_support;
   GError *err;
   int status;
   
@@ -446,12 +497,20 @@ main (int argc, char *argv[])
     gsm_set_verbose (TRUE);
   
   set_lang();
+  set_gtk1_theme_rcfile ();
 
   /* Initialize the i18n stuff */
   bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
+  /* Make sure the splash screen has the SM_CLIENT_ID
+   * property set so smproxy doesn't try and session
+   * manage us.
+   * See http://bugzilla.gnome.org/show_bug.cgi?id=118063
+   */
+  gdk_set_sm_client_id ("gnome-session-dummy-id");
+
   /* gnome_login_check() needs gtk initialized. however, these do
    * sanity checks for gnome stuff, so we don't want to initialize
    * gnome yet
@@ -513,6 +572,7 @@ main (int argc, char *argv[])
   splashing      = gconf_client_get_bool (gconf_client, SPLASH_SCREEN_KEY, NULL);
   autosave       = gconf_client_get_bool (gconf_client, AUTOSAVE_MODE_KEY, NULL);
   logout_prompt  = gconf_client_get_bool (gconf_client, LOGOUT_PROMPT_KEY, NULL);
+  a_t_support    = gconf_client_get_bool (gconf_client, ACCESSIBILITY_KEY, NULL);
 
   gconf_client_notify_add (gconf_client,
 			   AUTOSAVE_MODE_KEY,
@@ -565,6 +625,9 @@ main (int argc, char *argv[])
 
   start_session (the_session);
 
+  if (a_t_support) /* the ATs are happier if the session has started */
+    gsm_assistive_technologies_start ();
+
   gtk_main ();
 
   gsm_sound_logout ();
